@@ -9,15 +9,16 @@ use std::time::Duration;
 use byteorder::{LittleEndian, ReadBytesExt};
 
 #[derive(Clone, Debug, PartialEq, Default)]
+#[non_exhaustive]
 /// An APE file's audio properties
 pub struct ApeProperties {
-	version: u16,
-	duration: Duration,
-	overall_bitrate: u32,
-	audio_bitrate: u32,
-	sample_rate: u32,
-	bit_depth: u8,
-	channels: u8,
+	pub(crate) version: u16,
+	pub(crate) duration: Duration,
+	pub(crate) overall_bitrate: u32,
+	pub(crate) audio_bitrate: u32,
+	pub(crate) sample_rate: u32,
+	pub(crate) bit_depth: u8,
+	pub(crate) channels: u8,
 }
 
 impl From<ApeProperties> for FileProperties {
@@ -34,27 +35,6 @@ impl From<ApeProperties> for FileProperties {
 }
 
 impl ApeProperties {
-	/// Creates a new [`ApeProperties`]
-	pub const fn new(
-		version: u16,
-		duration: Duration,
-		overall_bitrate: u32,
-		audio_bitrate: u32,
-		sample_rate: u32,
-		bit_depth: u8,
-		channels: u8,
-	) -> Self {
-		Self {
-			version,
-			duration,
-			overall_bitrate,
-			audio_bitrate,
-			sample_rate,
-			bit_depth,
-			channels,
-		}
-	}
-
 	/// Duration
 	pub fn duration(&self) -> Duration {
 		self.duration
