@@ -1,5 +1,5 @@
 use super::FrameFlags;
-use crate::error::{LoftyError, Result};
+use crate::error::{Id3v2Error, Id3v2ErrorKind, Result};
 use crate::id3::v2::util::upgrade::{upgrade_v2, upgrade_v3};
 use crate::id3::v2::FrameID;
 
@@ -20,7 +20,8 @@ where
 		return Ok(None);
 	}
 
-	let id_str = std::str::from_utf8(&frame_header[..3]).map_err(|_| LoftyError::BadFrameID)?;
+	let id_str = std::str::from_utf8(&frame_header[..3])
+		.map_err(|_| Id3v2Error::new(Id3v2ErrorKind::BadFrameID))?;
 	let id = upgrade_v2(id_str).unwrap_or(id_str);
 
 	let frame_id = FrameID::new(id)?;
@@ -49,7 +50,8 @@ where
 		return Ok(None);
 	}
 
-	let id_str = std::str::from_utf8(&frame_header[..4]).map_err(|_| LoftyError::BadFrameID)?;
+	let id_str = std::str::from_utf8(&frame_header[..4])
+		.map_err(|_| Id3v2Error::new(Id3v2ErrorKind::BadFrameID))?;
 
 	let (id, size) = if synchsafe {
 		let size = crate::id3::v2::unsynch_u32(u32::from_be_bytes([
