@@ -4,11 +4,12 @@ use super::frame::id::FrameID;
 use super::frame::{Frame, FrameFlags, FrameValue};
 use super::util::text_utils::TextEncoding;
 use super::Id3v2Version;
-use crate::error::Result;
+use crate::error::{LoftyError, Result};
 use crate::id3::v2::frame::FrameRef;
+use crate::tag_traits::{Accessor, TagExt};
 use crate::types::item::{ItemKey, ItemValue, TagItem};
 use crate::types::picture::{Picture, PictureType};
-use crate::types::tag::{Accessor, Tag, TagType};
+use crate::types::tag::{Tag, TagType};
 
 use std::convert::TryInto;
 use std::fs::{File, OpenOptions};
@@ -103,6 +104,15 @@ impl_accessor!(
 	genre,        "TCON";
 );
 
+impl IntoIterator for Id3v2Tag {
+	type Item = Frame;
+	type IntoIter = std::vec::IntoIter<Frame>;
+
+	fn into_iter(self) -> Self::IntoIter {
+		self.frames.into_iter()
+	}
+}
+
 impl Default for Id3v2Tag {
 	fn default() -> Self {
 		Self {
@@ -144,11 +154,6 @@ impl Id3v2Tag {
 		self.frames.len()
 	}
 
-	/// Returns `true` if the tag contains no frames
-	pub fn is_empty(&self) -> bool {
-		self.frames.is_empty()
-	}
-
 	/// Gets a [`Frame`] from an id
 	///
 	/// NOTE: This is *not* case-sensitive
@@ -265,15 +270,15 @@ impl Id3v2Tag {
 	}
 }
 
-impl Id3v2Tag {
-	/// Writes the tag to a path
-	///
-	/// # Errors
-	///
-	/// * `path` does not exist
-	/// * See [`Id3v2Tag::write_to`]
-	pub fn write_to_path(&self, path: impl AsRef<Path>) -> Result<()> {
-		self.write_to(&mut OpenOptions::new().read(true).write(true).open(path)?)
+impl TagExt for Id3v2Tag {
+	type Err = LoftyError;
+
+	fn is_empty(&self) -> bool {
+		self.frames.is_empty()
+	}
+
+	fn save_to_path<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), Self::Err> {
+		self.save_to(&mut OpenOptions::new().read(true).write(true).open(path)?)
 	}
 
 	/// Writes the tag to a file
@@ -283,9 +288,12 @@ impl Id3v2Tag {
 	/// * Attempting to write the tag to a format that does not support it
 	/// * Attempting to write an encrypted frame without a valid method symbol or data length indicator
 	/// * Attempting to write an invalid [`FrameID`]/[`FrameValue`] pairing
-	pub fn write_to(&self, file: &mut File) -> Result<()> {
-		Id3v2TagRef::new(self.flags, self.frames.iter().filter_map(Frame::as_opt_ref))
-			.write_to(file)
+	fn save_to(&self, file: &mut File) -> std::result::Result<(), Self::Err> {
+		Id3v2TagRef {
+			flags: self.flags,
+			frames: self.frames.iter().filter_map(Frame::as_opt_ref),
+		}
+		.write_to(file)
 	}
 
 	/// Dumps the tag to a writer
@@ -293,18 +301,21 @@ impl Id3v2Tag {
 	/// # Errors
 	///
 	/// * [`std::io::Error`]
-	pub fn dump_to<W: Write>(&self, writer: &mut W) -> Result<()> {
-		Id3v2TagRef::new(self.flags, self.frames.iter().filter_map(Frame::as_opt_ref))
-			.dump_to(writer)
+	/// * [`ErrorKind::TooMuchData`](crate::error::ErrorKind::TooMuchData)
+	fn dump_to<W: Write>(&self, writer: &mut W) -> std::result::Result<(), Self::Err> {
+		Id3v2TagRef {
+			flags: self.flags,
+			frames: self.frames.iter().filter_map(Frame::as_opt_ref),
+		}
+		.dump_to(writer)
 	}
-}
 
-impl IntoIterator for Id3v2Tag {
-	type Item = Frame;
-	type IntoIter = std::vec::IntoIter<Frame>;
+	fn remove_from_path<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), Self::Err> {
+		TagType::Id3v2.remove_from_path(path)
+	}
 
-	fn into_iter(self) -> Self::IntoIter {
-		self.frames.into_iter()
+	fn remove_from(&self, file: &mut File) -> std::result::Result<(), Self::Err> {
+		TagType::Id3v2.remove_from(file)
 	}
 }
 
@@ -414,16 +425,10 @@ pub(crate) struct Id3v2TagRef<'a, I: Iterator<Item = FrameRef<'a>> + 'a> {
 pub(crate) fn tag_frames(tag: &Tag) -> impl Iterator<Item = FrameRef<'_>> + '_ {
 	tag.items()
 		.iter()
-		.map(TryInto::<FrameRef>::try_into)
+		.map(TryInto::<FrameRef<'_>>::try_into)
 		.filter_map(Result::ok)
 }
 
-impl<'a, I: Iterator<Item = FrameRef<'a>> + 'a> Id3v2TagRef<'a, I> {
-	pub(crate) fn new(flags: Id3v2TagFlags, frames: I) -> Self {
-		Self { flags, frames }
-	}
-}
-
 impl<'a, I: Iterator<Item = FrameRef<'a>> + 'a> Id3v2TagRef<'a, I> {
 	pub(crate) fn write_to(&mut self, file: &mut File) -> Result<()> {
 		super::write::write_id3v2(file, self)
@@ -444,7 +449,7 @@ mod tests {
 		LanguageFrame, TextEncoding,
 	};
 	use crate::tag_utils::test_utils::read_path;
-	use crate::{MimeType, Picture, PictureType, Tag, TagType};
+	use crate::{MimeType, Picture, PictureType, Tag, TagExt, TagType};
 
 	fn read_tag(path: &str) -> Id3v2Tag {
 		let tag_bytes = crate::tag_utils::test_utils::read_path(path);
@@ -749,4 +754,20 @@ mod tests {
 
 		assert_eq!(tag, parsed_tag);
 	}
+
+	#[test]
+	fn id3v24_footer() {
+		let mut tag = create_full_test_tag(Id3v2Version::V4);
+		tag.flags.footer = true;
+
+		let mut writer = Vec::new();
+		tag.dump_to(&mut writer).unwrap();
+
+		let mut reader = &mut &writer[..];
+
+		let header = read_id3v2_header(&mut reader).unwrap();
+		assert!(crate::id3::v2::read::parse_id3v2(reader, header).is_ok());
+
+		assert_eq!(writer[3..10], writer[writer.len() - 7..])
+	}
 }
