@@ -1,7 +1,8 @@
 use crate::error::{ErrorKind, LoftyError, Result};
 use crate::iff::chunk::Chunks;
+use crate::tag_traits::{Accessor, TagExt};
 use crate::types::item::{ItemKey, ItemValue, TagItem};
-use crate::types::tag::{Accessor, Tag, TagType};
+use crate::types::tag::{Tag, TagType};
 
 use std::convert::TryFrom;
 use std::fs::{File, OpenOptions};
@@ -111,51 +112,63 @@ impl AiffTextChunks {
 	pub fn remove_copyright(&mut self) {
 		self.copyright = None
 	}
+}
+
+impl TagExt for AiffTextChunks {
+	type Err = LoftyError;
+
+	fn is_empty(&self) -> bool {
+		matches!(
+			self,
+			AiffTextChunks {
+				name: None,
+				author: None,
+				copyright: None,
+				annotations: None,
+				comments: None
+			}
+		)
+	}
 
 	/// Writes the tag to a path
 	///
 	/// # Errors
 	///
 	/// * `path` does not exist
-	/// * See [`AiffTextChunks::write_to`]
-	pub fn write_to_path(&self, path: impl AsRef<Path>) -> Result<()> {
-		self.write_to(&mut OpenOptions::new().read(true).write(true).open(path)?)
+	/// * See [`AiffTextChunks::save_to`]
+	fn save_to_path<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), Self::Err> {
+		self.save_to(&mut OpenOptions::new().read(true).write(true).open(path)?)
 	}
 
-	/// Writes the tag to a file
-	///
-	/// # Errors
-	///
-	/// * Attempting to write the tag to a format that does not support it
-	pub fn write_to(&self, file: &mut File) -> Result<()> {
-		AiffTextChunksRef::new(
-			self.name.as_deref(),
-			self.author.as_deref(),
-			self.copyright.as_deref(),
-			self.annotations.as_deref(),
-			self.comments.as_deref(),
-		)
+	fn save_to(&self, file: &mut File) -> std::result::Result<(), Self::Err> {
+		AiffTextChunksRef {
+			name: self.name.as_deref(),
+			author: self.author.as_deref(),
+			copyright: self.copyright.as_deref(),
+			annotations: self.annotations.as_deref(),
+			comments: self.comments.as_deref(),
+		}
 		.write_to(file)
 	}
 
-	/// Dumps the tag to a writer
-	///
-	/// This will only write the metadata chunks, it will not create a usable
-	/// file.
-	///
-	/// # Errors
-	///
-	/// * [`std::io::Error`]
-	pub fn dump_to<W: Write>(&self, writer: &mut W) -> Result<()> {
-		AiffTextChunksRef::new(
-			self.name.as_deref(),
-			self.author.as_deref(),
-			self.copyright.as_deref(),
-			self.annotations.as_deref(),
-			self.comments.as_deref(),
-		)
+	fn dump_to<W: Write>(&self, writer: &mut W) -> std::result::Result<(), Self::Err> {
+		AiffTextChunksRef {
+			name: self.name.as_deref(),
+			author: self.author.as_deref(),
+			copyright: self.copyright.as_deref(),
+			annotations: self.annotations.as_deref(),
+			comments: self.comments.as_deref(),
+		}
 		.dump_to(writer)
 	}
+
+	fn remove_from_path<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), Self::Err> {
+		TagType::AiffText.remove_from_path(path)
+	}
+
+	fn remove_from(&self, file: &mut File) -> std::result::Result<(), Self::Err> {
+		TagType::AiffText.remove_from(file)
+	}
 }
 
 impl From<AiffTextChunks> for Tag {
@@ -235,22 +248,6 @@ where
 	T: AsRef<str>,
 	AI: IntoIterator<Item = T>,
 {
-	pub(crate) fn new(
-		name: Option<&'a str>,
-		author: Option<&'a str>,
-		copyright: Option<&'a str>,
-		annotations: Option<AI>,
-		comments: Option<&'a [Comment]>,
-	) -> AiffTextChunksRef<'a, T, AI> {
-		AiffTextChunksRef {
-			name,
-			author,
-			copyright,
-			annotations,
-			comments,
-		}
-	}
-
 	pub(crate) fn write_to(self, file: &mut File) -> Result<()> {
 		AiffTextChunksRef::write_to_inner(file, self)
 	}
@@ -262,7 +259,7 @@ where
 		Ok(())
 	}
 
-	fn create_text_chunks(tag: &mut AiffTextChunksRef<T, AI>) -> Result<Vec<u8>> {
+	fn create_text_chunks(tag: &mut AiffTextChunksRef<'_, T, AI>) -> Result<Vec<u8>> {
 		fn write_chunk(writer: &mut Vec<u8>, key: &str, value: Option<&str>) {
 			if let Some(val) = value {
 				if let Ok(len) = u32::try_from(val.len()) {
@@ -342,14 +339,14 @@ where
 		Ok(text_chunks)
 	}
 
-	fn write_to_inner(data: &mut File, mut tag: AiffTextChunksRef<T, AI>) -> Result<()> {
-		super::read::verify_aiff(data)?;
+	fn write_to_inner(data: &mut File, mut tag: AiffTextChunksRef<'_, T, AI>) -> Result<()> {
+		let file_size = super::read::verify_aiff(data)?;
 
 		let text_chunks = Self::create_text_chunks(&mut tag)?;
 
 		let mut chunks_remove = Vec::new();
 
-		let mut chunks = Chunks::<BigEndian>::new();
+		let mut chunks = Chunks::<BigEndian>::new(file_size);
 
 		while chunks.next(data).is_ok() {
 			match &chunks.fourcc {
@@ -386,7 +383,7 @@ where
 			chunks_remove.sort_unstable();
 			chunks_remove.reverse();
 
-			let first = chunks_remove.pop().unwrap();
+			let first = chunks_remove.pop().unwrap(); // Infallible
 
 			for (s, e) in &chunks_remove {
 				file_bytes.drain(*s as usize..*e as usize);
@@ -409,7 +406,7 @@ where
 #[cfg(test)]
 mod tests {
 	use crate::iff::{AiffTextChunks, Comment};
-	use crate::{ItemKey, ItemValue, Tag, TagItem, TagType};
+	use crate::{ItemKey, ItemValue, Tag, TagExt, TagItem, TagType};
 
 	use std::io::Cursor;
 
@@ -456,7 +453,9 @@ mod tests {
 			.unwrap();
 
 		// Create a fake AIFF signature
-		let mut writer = vec![b'F', b'O', b'R', b'M', 0, 0, 0, 0, b'A', b'I', b'F', b'F'];
+		let mut writer = vec![
+			b'F', b'O', b'R', b'M', 0, 0, 0, 0xC6, b'A', b'I', b'F', b'F',
+		];
 		parsed_tag.dump_to(&mut writer).unwrap();
 
 		let temp_parsed_tag = super::super::read::read_from(&mut Cursor::new(writer), false)
