@@ -1,9 +1,10 @@
 pub(super) mod read;
 mod write;
 
-use crate::error::Result;
+use crate::error::{LoftyError, Result};
+use crate::tag_traits::{Accessor, TagExt};
 use crate::types::item::{ItemKey, ItemValue, TagItem};
-use crate::types::tag::{Accessor, Tag, TagType};
+use crate::types::tag::{Tag, TagType};
 
 use std::fs::{File, OpenOptions};
 use std::io::Write;
@@ -73,7 +74,7 @@ impl RiffInfoList {
 	///
 	/// This will case-insensitively replace any item with the same key
 	pub fn insert(&mut self, key: String, value: String) {
-		if valid_key(key.as_str()) {
+		if read::verify_key(key.as_str()) {
 			self.items
 				.iter()
 				.position(|(k, _)| k.eq_ignore_ascii_case(key.as_str()))
@@ -98,36 +99,33 @@ impl RiffInfoList {
 	}
 }
 
-impl RiffInfoList {
-	/// Writes the tag to a path
-	///
-	/// # Errors
-	///
-	/// * `path` does not exist
-	/// * See [`RiffInfoList::write_to`]
-	pub fn write_to_path(&self, path: impl AsRef<Path>) -> Result<()> {
-		self.write_to(&mut OpenOptions::new().read(true).write(true).open(path)?)
+impl TagExt for RiffInfoList {
+	type Err = LoftyError;
+
+	fn is_empty(&self) -> bool {
+		self.items.is_empty()
 	}
 
-	/// Writes the tag to a file
-	///
-	/// # Errors
-	///
-	/// * Attempting to write the tag to a format that does not support it
-	pub fn write_to(&self, file: &mut File) -> Result<()> {
-		Into::<RiffInfoListRef>::into(self).write_to(file)
+	fn save_to_path<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), Self::Err> {
+		self.save_to(&mut OpenOptions::new().read(true).write(true).open(path)?)
 	}
 
-	/// Dumps the tag to a writer
-	///
-	/// This will only write the LIST chunk, it will not create a usable
-	/// file.
-	///
-	/// # Errors
-	///
-	/// * [`std::io::Error`]
-	pub fn dump_to<W: Write>(&self, writer: &mut W) -> Result<()> {
-		Into::<RiffInfoListRef>::into(self).dump_to(writer)
+	fn save_to(&self, file: &mut File) -> std::result::Result<(), Self::Err> {
+		RiffInfoListRef::new(self.items.iter().map(|(k, v)| (k.as_str(), v.as_str())))
+			.write_to(file)
+	}
+
+	fn dump_to<W: Write>(&self, writer: &mut W) -> std::result::Result<(), Self::Err> {
+		RiffInfoListRef::new(self.items.iter().map(|(k, v)| (k.as_str(), v.as_str())))
+			.dump_to(writer)
+	}
+
+	fn remove_from_path<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), Self::Err> {
+		TagType::RiffInfo.remove_from_path(path)
+	}
+
+	fn remove_from(&self, file: &mut File) -> std::result::Result<(), Self::Err> {
+		TagType::RiffInfo.remove_from(file)
 	}
 }
 
@@ -154,24 +152,18 @@ impl From<Tag> for RiffInfoList {
 
 		for item in input.items {
 			if let ItemValue::Text(val) | ItemValue::Locator(val) = item.item_value {
-				let item_key = match item.item_key {
+				match item.item_key {
 					ItemKey::Unknown(unknown) => {
-						if unknown.len() == 4 && unknown.is_ascii() {
-							unknown.to_string()
-						} else {
-							continue;
+						if read::verify_key(&unknown) {
+							riff_info.items.push((unknown, val))
 						}
 					},
 					k => {
 						if let Some(key) = k.map_key(TagType::RiffInfo, false) {
-							key.to_string()
-						} else {
-							continue;
+							riff_info.items.push((key.to_string(), val))
 						}
 					},
-				};
-
-				riff_info.items.push((item_key, val))
+				}
 			}
 		}
 
@@ -179,39 +171,21 @@ impl From<Tag> for RiffInfoList {
 	}
 }
 
-pub(crate) struct RiffInfoListRef<'a> {
-	items: Box<dyn Iterator<Item = (&'a str, &'a String)> + 'a>,
+pub(crate) struct RiffInfoListRef<'a, I>
+where
+	I: Iterator<Item = (&'a str, &'a str)>,
+{
+	pub(crate) items: I,
 }
 
-impl<'a> Into<RiffInfoListRef<'a>> for &'a RiffInfoList {
-	fn into(self) -> RiffInfoListRef<'a> {
-		RiffInfoListRef {
-			items: Box::new(self.items.iter().map(|(k, v)| (k.as_str(), v))),
-		}
+impl<'a, I> RiffInfoListRef<'a, I>
+where
+	I: Iterator<Item = (&'a str, &'a str)>,
+{
+	pub(crate) fn new(items: I) -> RiffInfoListRef<'a, I> {
+		RiffInfoListRef { items }
 	}
-}
 
-impl<'a> Into<RiffInfoListRef<'a>> for &'a Tag {
-	fn into(self) -> RiffInfoListRef<'a> {
-		RiffInfoListRef {
-			items: Box::new(self.items.iter().filter_map(|i| {
-				if let ItemValue::Text(val) | ItemValue::Locator(val) = &i.item_value {
-					let item_key = i.key().map_key(TagType::RiffInfo, true).unwrap();
-
-					if item_key.len() == 4 && item_key.is_ascii() {
-						Some((item_key, val))
-					} else {
-						None
-					}
-				} else {
-					None
-				}
-			})),
-		}
-	}
-}
-
-impl<'a> RiffInfoListRef<'a> {
 	pub(crate) fn write_to(&mut self, file: &mut File) -> Result<()> {
 		write::write_riff_info(file, self)
 	}
@@ -226,15 +200,28 @@ impl<'a> RiffInfoListRef<'a> {
 	}
 }
 
-fn valid_key(key: &str) -> bool {
-	key.len() == 4 && key.is_ascii()
+pub(crate) fn tagitems_into_riff(items: &[TagItem]) -> impl Iterator<Item = (&str, &str)> {
+	items.iter().filter_map(|i| {
+		let item_key = i.key().map_key(TagType::RiffInfo, true);
+
+		match (item_key, i.value()) {
+			(Some(key), ItemValue::Text(val) | ItemValue::Locator(val))
+				if read::verify_key(key) =>
+			{
+				Some((key, val.as_str()))
+			},
+			_ => None,
+		}
+	})
 }
 
 #[cfg(test)]
 mod tests {
 	use crate::iff::RiffInfoList;
-	use crate::{Tag, TagType};
+	use crate::{Tag, TagExt, TagType};
 
+	use crate::iff::chunk::Chunks;
+	use byteorder::LittleEndian;
 	use std::io::Cursor;
 
 	#[test]
@@ -253,6 +240,7 @@ mod tests {
 
 		super::read::parse_riff_info(
 			&mut Cursor::new(&tag[..]),
+			&mut Chunks::<LittleEndian>::new(tag.len() as u32),
 			(tag.len() - 1) as u64,
 			&mut parsed_tag,
 		)
@@ -268,6 +256,7 @@ mod tests {
 
 		super::read::parse_riff_info(
 			&mut Cursor::new(&tag[..]),
+			&mut Chunks::<LittleEndian>::new(tag.len() as u32),
 			(tag.len() - 1) as u64,
 			&mut parsed_tag,
 		)
@@ -281,6 +270,7 @@ mod tests {
 		// Remove the LIST....INFO from the tag
 		super::read::parse_riff_info(
 			&mut Cursor::new(&writer[12..]),
+			&mut Chunks::<LittleEndian>::new(tag.len() as u32),
 			(tag.len() - 13) as u64,
 			&mut temp_parsed_tag,
 		)
@@ -296,8 +286,13 @@ mod tests {
 		let mut reader = std::io::Cursor::new(&tag_bytes[..]);
 		let mut riff_info = RiffInfoList::default();
 
-		super::read::parse_riff_info(&mut reader, (tag_bytes.len() - 1) as u64, &mut riff_info)
-			.unwrap();
+		super::read::parse_riff_info(
+			&mut reader,
+			&mut Chunks::<LittleEndian>::new(tag_bytes.len() as u32),
+			(tag_bytes.len() - 1) as u64,
+			&mut riff_info,
+		)
+		.unwrap();
 
 		let tag: Tag = riff_info.into();
 
