@@ -6,16 +6,20 @@
 //!
 //! # Supported Formats
 //!
-//! | File Format | Extensions                                      | Read | Write | Metadata Format(s)                            |
-//! |-------------|-------------------------------------------------|------|-------|-----------------------------------------------|
-//! | APE         | `ape`                                           |**X** |**X**  |`APEv2`, `APEv1`, `ID3v2` (Read only), `ID3v1` |
-//! | AIFF        | `aiff`, `aif`                                   |**X** |**X**  |`ID3v2`, `Text Chunks`                         |
-//! | FLAC        | `flac`                                          |**X** |**X**  |`Vorbis Comments`                              |
-//! | MP3         | `mp3`                                           |**X** |**X**  |`ID3v2`, `ID3v1`, `APEv2`, `APEv1`             |
-//! | MP4         | `mp4`, `m4a`, `m4b`, `m4p`, `m4r`, `m4v`, `3gp` |**X** |**X**  |`iTunes-style ilst`                            |
-//! | Opus        | `opus`                                          |**X** |**X**  |`Vorbis Comments`                              |
-//! | Ogg Vorbis  | `ogg`                                           |**X** |**X**  |`Vorbis Comments`                              |
-//! | WAV         | `wav`, `wave`                                   |**X** |**X**  |`ID3v2`, `RIFF INFO`                           |
+//!
+//! | File Format | Metadata Format(s)                  |
+//! |-------------|-------------------------------------|
+//! | Ape         | `APEv2`, `APEv1`, `ID3v2`*, `ID3v1` |
+//! | AIFF        | `ID3v2`, `Text Chunks`              |
+//! | FLAC        | `Vorbis Comments`                   |
+//! | MP3         | `ID3v2`, `ID3v1`, `APEv2`, `APEv1`  |
+//! | MP4         | `iTunes-style ilst`                 |
+//! | Opus        | `Vorbis Comments`                   |
+//! | Ogg Vorbis  | `Vorbis Comments`                   |
+//! | Speex       | `Vorbis Comments`                   |
+//! | WAV         | `ID3v2`, `RIFF INFO`                |
+//!
+//! \* APE does not officially support ID3v2, so the tag will be **read only**
 //!
 //! # Examples
 //!
@@ -34,11 +38,11 @@
 //! // First, create a probe.
 //! // This will guess the format from the extension
 //! // ("mp3" in this case), but we can guess from the content if we want to.
-//! let tagged_file = read_from_path("tests/files/assets/a.mp3", false)?;
+//! let tagged_file = read_from_path("tests/files/assets/minimal/full_test.mp3", false)?;
 //!
 //! // Let's guess the format from the content just in case.
 //! // This is not necessary in this case!
-//! let tagged_file2 = Probe::open("tests/files/assets/a.mp3")?
+//! let tagged_file2 = Probe::open("tests/files/assets/minimal/full_test.mp3")?
 //! 	.guess_file_type()?
 //! 	.read(false)?;
 //! # Ok(())
@@ -54,7 +58,7 @@
 //! use std::fs::File;
 //!
 //! // Let's read from an open file
-//! let mut file = File::open("tests/files/assets/a.mp3")?;
+//! let mut file = File::open("tests/files/assets/minimal/full_test.mp3")?;
 //!
 //! // Here, we have to guess the file type prior to reading
 //! let tagged_file = read_from(&mut file, false)?;
@@ -69,14 +73,14 @@
 //! # fn main() -> Result<(), LoftyError> {
 //! use lofty::read_from_path;
 //!
-//! let tagged_file = read_from_path("tests/files/assets/a.mp3", false)?;
+//! let tagged_file = read_from_path("tests/files/assets/minimal/full_test.mp3", false)?;
 //!
 //! // Get the primary tag (ID3v2 in this case)
-//! let id3v2 = tagged_file.primary_tag().unwrap();
+//! let id3v2 = tagged_file.primary_tag();
 //!
 //! // If the primary tag doesn't exist, or the tag types
 //! // don't matter, the first tag can be retrieved
-//! let unknown_first_tag = tagged_file.first_tag().unwrap();
+//! let unknown_first_tag = tagged_file.first_tag();
 //! # Ok(())
 //! # }
 //! ```
@@ -90,7 +94,7 @@
 //! use lofty::{AudioFile, TagType};
 //! use std::fs::File;
 //!
-//! let mut file_content = File::open("tests/files/assets/a.mp3")?;
+//! let mut file_content = File::open("tests/files/assets/minimal/full_test.mp3")?;
 //!
 //! // We are expecting an MP3 file
 //! let mpeg_file = Mp3File::read_from(&mut file_content, true)?;
@@ -98,8 +102,8 @@
 //! assert_eq!(mpeg_file.properties().channels(), 2);
 //!
 //! // Here we have a file with multiple tags
-//! assert!(mpeg_file.contains_tag_type(&TagType::Id3v2));
-//! assert!(mpeg_file.contains_tag_type(&TagType::Ape));
+//! assert!(mpeg_file.contains_tag_type(TagType::Id3v2));
+//! assert!(mpeg_file.contains_tag_type(TagType::Ape));
 //! # Ok(())
 //! # }
 //! ```
@@ -124,12 +128,16 @@
 //!
 //! All formats have their own quirks that may produce unexpected results between conversions.
 //! Be sure to read the module documentation of each format to see important notes and warnings.
+#![forbid(clippy::dbg_macro, clippy::string_to_string)]
 #![deny(
 	clippy::pedantic,
 	clippy::all,
 	missing_docs,
-	rustdoc::broken_intra_doc_links
+	rustdoc::broken_intra_doc_links,
+	rust_2018_idioms
 )]
+// TODO: This had multiple FPs right now, remove this when it is fixed
+#![allow(clippy::needless_borrow)]
 #![allow(
 	clippy::too_many_lines,
 	clippy::cast_precision_loss,
@@ -145,20 +153,23 @@
 	clippy::new_without_default,
 	clippy::from_over_into,
 	clippy::upper_case_acronyms,
-	clippy::too_many_arguments,
 	clippy::single_match_else,
 	clippy::similar_names,
-	clippy::tabs_in_doc_comments
+	clippy::tabs_in_doc_comments,
+	clippy::len_without_is_empty
 )]
+#![cfg_attr(docsrs, feature(doc_auto_cfg))]
 
 pub mod ape;
 pub mod error;
 pub mod id3;
 pub mod iff;
+pub(crate) mod macros;
 pub mod mp3;
 pub mod mp4;
 pub mod ogg;
 mod probe;
+mod tag_traits;
 pub(crate) mod tag_utils;
 mod types;
 
@@ -170,7 +181,9 @@ pub use crate::types::file::{AudioFile, FileType, TaggedFile};
 pub use crate::types::item::{ItemKey, ItemValue, TagItem};
 pub use crate::types::picture::{MimeType, Picture, PictureType};
 pub use crate::types::properties::FileProperties;
-pub use crate::types::tag::{Accessor, Tag, TagType};
+pub use crate::types::tag::{Tag, TagType};
+
+pub use crate::tag_traits::{Accessor, TagExt};
 
 #[cfg(feature = "vorbis_comments")]
 pub use crate::types::picture::PictureInformation;
