@@ -1,9 +1,9 @@
 use super::header::{search_for_frame_sync, Header, XingHeader};
 use super::{Mp3File, Mp3Properties};
 use crate::ape::constants::APE_PREAMBLE;
+use crate::ape::header::read_ape_header;
 #[cfg(feature = "ape")]
 use crate::ape::tag::read::read_ape_tag;
-use crate::ape::tag::read_ape_header;
 use crate::error::{FileDecodingError, Result};
 #[cfg(feature = "id3v2")]
 use crate::id3::v2::read::parse_id3v2;
@@ -83,7 +83,7 @@ where
 			},
 			// Tags might be followed by junk bytes before the first MP3 frame begins
 			_ => {
-				// seek back the length of the temporary header buffer, to include them
+				// Seek back the length of the temporary header buffer, to include them
 				// in the frame sync search
 				#[allow(clippy::neg_multiply)]
 				let start_of_search_area = reader.seek(SeekFrom::Current(-1 * header.len() as i64))?;
@@ -102,7 +102,8 @@ where
 					// We have found the first frame
 					break;
 				}
-				// the search for sync bits was unsuccessful
+
+				// The search for sync bits was unsuccessful
 				return Err(FileDecodingError::new(
 					FileType::MP3,
 					"File contains an invalid frame",
@@ -163,7 +164,7 @@ where
 		let mut xing_reader = [0; 32];
 		reader.read_exact(&mut xing_reader)?;
 
-		let xing_header = XingHeader::read(&mut &xing_reader[..]).ok();
+		let xing_header = XingHeader::read(&mut &xing_reader[..])?;
 
 		super::properties::read_properties(
 			(first_frame_header, first_frame_offset),
