@@ -14,14 +14,19 @@ use std::fs::File;
 pub(crate) fn write_to(data: &mut File, tag: &Tag) -> Result<()> {
 	match tag.tag_type() {
 		#[cfg(feature = "ape")]
-		TagType::Ape => Into::<ape_tag::ApeTagRef>::into(tag).write_to(data),
+		TagType::Ape => ape_tag::ApeTagRef {
+			read_only: false,
+			items: ape_tag::tagitems_into_ape(tag.items()),
+		}
+		.write_to(data),
 		#[cfg(feature = "id3v1")]
-		TagType::Id3v1 => Into::<v1::tag::Id3v1TagRef>::into(tag).write_to(data),
+		TagType::Id3v1 => Into::<v1::tag::Id3v1TagRef<'_>>::into(tag).write_to(data),
 		#[cfg(feature = "id3v2")]
-		TagType::Id3v2 => {
-			v2::tag::Id3v2TagRef::new(v2::Id3v2TagFlags::default(), v2::tag::tag_frames(tag))
-				.write_to(data)
-		},
+		TagType::Id3v2 => v2::tag::Id3v2TagRef {
+			flags: v2::Id3v2TagFlags::default(),
+			frames: v2::tag::tag_frames(tag),
+		}
+		.write_to(data),
 		_ => Err(LoftyError::new(ErrorKind::UnsupportedTag)),
 	}
 }
