@@ -1,13 +1,15 @@
 pub(super) mod atom;
+pub(super) mod constants;
 pub(super) mod read;
 pub(crate) mod write;
 
 use super::AtomIdent;
-use crate::error::Result;
+use crate::error::{LoftyError, Result};
+use crate::tag_traits::{Accessor, TagExt};
 use crate::types::item::{ItemKey, ItemValue, TagItem};
 use crate::types::picture::{Picture, PictureType};
-use crate::types::tag::{Accessor, Tag, TagType};
-use atom::{Atom, AtomData, AtomDataRef, AtomIdentRef, AtomRef};
+use crate::types::tag::{Tag, TagType};
+use atom::{AdvisoryRating, Atom, AtomData, AtomDataRef, AtomIdentRef, AtomRef};
 
 use std::convert::TryInto;
 use std::fs::{File, OpenOptions};
@@ -66,15 +68,17 @@ macro_rules! impl_accessor {
 ///
 /// ### To `Tag`
 ///
-/// When converting to [`Tag`], only atoms with a value of [`AtomData::UTF8`] and [`AtomData::UTF16`], as
-/// well as pictures, will be preserved.
+/// When converting to [`Tag`], only atoms with a value of [`AtomData::UTF8`] and [`AtomData::UTF16`],
+/// with the exception of the `trkn` and `disk` atoms, as well as pictures, will be preserved.
 ///
 /// Do note, all pictures will be [`PictureType::Other`](crate::PictureType::Other)
 ///
 /// ### From `Tag`
 ///
 /// When converting from [`Tag`], only items with a value of [`ItemValue::Text`](crate::ItemValue::Text), as
-/// well as pictures, will be preserved
+/// well as pictures, will be preserved.
+///
+/// An attempt will be made to create the `TrackNumber/TrackTotal` (trkn) and `DiscNumber/DiscTotal` (disk) pairs.
 pub struct Ilst {
 	pub(crate) atoms: Vec<Atom>,
 }
@@ -141,6 +145,31 @@ impl Ilst {
 			.retain(|a| !matches!(a.data(), AtomData::Picture(_)))
 	}
 
+	/// Returns the parental advisory rating according to the `rtng` atom
+	pub fn advisory_rating(&self) -> Option<AdvisoryRating> {
+		if let Some(Atom { data, .. }) = self.atom(&AtomIdent::Fourcc(*b"rtng")) {
+			let rating = match data {
+				AtomData::SignedInteger(si) => *si as u8,
+				AtomData::Unknown { data: c, .. } if !c.is_empty() => c[0],
+				_ => return None,
+			};
+
+			return Some(AdvisoryRating::from(rating));
+		}
+
+		None
+	}
+
+	/// Sets the advisory rating
+	pub fn set_advisory_rating(&mut self, advisory_rating: AdvisoryRating) {
+		let byte = advisory_rating.as_u8();
+
+		self.replace_atom(Atom {
+			ident: AtomIdent::Fourcc(*b"rtng"),
+			data: AtomData::SignedInteger(i32::from(byte)),
+		})
+	}
+
 	/// Returns the track number
 	pub fn track_number(&self) -> Option<u16> {
 		self.extract_number(*b"trkn", 4)
@@ -179,36 +208,32 @@ impl Ilst {
 	}
 }
 
-impl Ilst {
-	/// Writes the tag to a path
-	///
-	/// # Errors
-	///
-	/// * `path` does not exist
-	/// * See [`Ilst::write_to`]
-	pub fn write_to_path(&self, path: impl AsRef<Path>) -> Result<()> {
-		self.write_to(&mut OpenOptions::new().read(true).write(true).open(path)?)
+impl TagExt for Ilst {
+	type Err = LoftyError;
+
+	fn is_empty(&self) -> bool {
+		self.atoms.is_empty()
 	}
 
-	/// Writes the tag to a file
-	///
-	/// # Errors
-	///
-	/// * Attempting to write the tag to a format that does not support it
-	pub fn write_to(&self, file: &mut File) -> Result<()> {
-		Into::<IlstRef>::into(self).write_to(file)
+	fn save_to_path<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), Self::Err> {
+		let mut f = OpenOptions::new().read(true).write(true).open(path)?;
+		self.save_to(&mut f)
 	}
 
-	/// Dumps the tag to a writer
-	///
-	/// This will only write the ilst atom, it will not create a usable
-	/// file.
-	///
-	/// # Errors
-	///
-	/// * [`std::io::Error`]
-	pub fn dump_to<W: Write>(&self, writer: &mut W) -> Result<()> {
-		Into::<IlstRef>::into(self).dump_to(writer)
+	fn save_to(&self, file: &mut File) -> std::result::Result<(), Self::Err> {
+		Into::<IlstRef<'_>>::into(self).write_to(file)
+	}
+
+	fn dump_to<W: Write>(&self, writer: &mut W) -> std::result::Result<(), Self::Err> {
+		Into::<IlstRef<'_>>::into(self).dump_to(writer)
+	}
+
+	fn remove_from_path<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), Self::Err> {
+		TagType::Mp4Ilst.remove_from_path(path)
+	}
+
+	fn remove_from(&self, file: &mut File) -> std::result::Result<(), Self::Err> {
+		TagType::Mp4Ilst.remove_from(file)
 	}
 }
 
@@ -224,29 +249,28 @@ impl From<Ilst> for Tag {
 					continue;
 				},
 				// We have to special case track/disc numbers since they are stored together
-				AtomData::Unknown { code: 0, data } if data.len() >= 6 => match atom.ident {
-					AtomIdent::Fourcc(ref fourcc) => match fourcc {
-						b"trkn" => {
-							let current = u16::from_be_bytes([data[2], data[3]]);
-							let total = u16::from_be_bytes([data[4], data[5]]);
+				AtomData::Unknown { code: 0, data } if data.len() >= 6 => {
+					if let AtomIdent::Fourcc(ref fourcc) = atom.ident {
+						match fourcc {
+							b"trkn" => {
+								let current = u16::from_be_bytes([data[2], data[3]]);
+								let total = u16::from_be_bytes([data[4], data[5]]);
 
-							tag.insert_text(ItemKey::TrackNumber, current.to_string());
-							tag.insert_text(ItemKey::TrackTotal, total.to_string());
+								tag.insert_text(ItemKey::TrackNumber, current.to_string());
+								tag.insert_text(ItemKey::TrackTotal, total.to_string());
+							},
+							b"disk" => {
+								let current = u16::from_be_bytes([data[2], data[3]]);
+								let total = u16::from_be_bytes([data[4], data[5]]);
 
-							continue;
-						},
-						b"disk" => {
-							let current = u16::from_be_bytes([data[2], data[3]]);
-							let total = u16::from_be_bytes([data[4], data[5]]);
+								tag.insert_text(ItemKey::DiscNumber, current.to_string());
+								tag.insert_text(ItemKey::DiscTotal, total.to_string());
+							},
+							_ => {},
+						}
+					}
 
-							tag.insert_text(ItemKey::DiscNumber, current.to_string());
-							tag.insert_text(ItemKey::DiscTotal, total.to_string());
-
-							continue;
-						},
-						_ => continue,
-					},
-					_ => continue,
+					continue;
 				},
 				_ => continue,
 			};
@@ -289,7 +313,7 @@ impl From<Tag> for Ilst {
 						ident: AtomIdent::Fourcc(ident),
 						data: AtomData::Unknown {
 							code: 0,
-							data: vec![0, 0, current[0], current[1], total[0], total[1]],
+							data: vec![0, 0, current[0], current[1], total[0], total[1], 0, 0],
 						},
 					})
 				},
@@ -386,7 +410,7 @@ impl<'a> Into<IlstRef<'a>> for &'a Tag {
 	}
 }
 
-fn item_key_to_ident(key: &ItemKey) -> Option<AtomIdentRef> {
+fn item_key_to_ident(key: &ItemKey) -> Option<AtomIdentRef<'_>> {
 	key.map_key(TagType::Mp4Ilst, true).and_then(|ident| {
 		if ident.starts_with("----") {
 			let mut split = ident.split(':');
@@ -415,8 +439,18 @@ fn item_key_to_ident(key: &ItemKey) -> Option<AtomIdentRef> {
 
 #[cfg(test)]
 mod tests {
-	use crate::mp4::{Atom, AtomData, AtomIdent, Ilst};
-	use crate::{ItemKey, Tag, TagType};
+	use crate::mp4::{AdvisoryRating, Atom, AtomData, AtomIdent, Ilst};
+	use crate::{ItemKey, Tag, TagExt, TagType};
+
+	fn read_ilst(path: &str) -> Ilst {
+		let tag = crate::tag_utils::test_utils::read_path(path);
+		super::read::parse_ilst(&mut &tag[..], tag.len() as u64).unwrap()
+	}
+
+	fn verify_atom(ilst: &Ilst, ident: [u8; 4], data: &AtomData) {
+		let atom = ilst.atom(&AtomIdent::Fourcc(ident)).unwrap();
+		assert_eq!(atom.data(), data);
+	}
 
 	#[test]
 	fn parse_ilst() {
@@ -481,8 +515,7 @@ mod tests {
 
 	#[test]
 	fn ilst_re_read() {
-		let tag = crate::tag_utils::test_utils::read_path("tests/tags/assets/test.ilst");
-		let parsed_tag = super::read::parse_ilst(&mut &tag[..], tag.len() as u64).unwrap();
+		let parsed_tag = read_ilst("tests/tags/assets/test.ilst");
 
 		let mut writer = Vec::new();
 		parsed_tag.dump_to(&mut writer).unwrap();
@@ -510,11 +543,6 @@ mod tests {
 
 	#[test]
 	fn tag_to_ilst() {
-		fn verify_atom(ilst: &Ilst, ident: [u8; 4], data: &AtomData) {
-			let atom = ilst.atom(&AtomIdent::Fourcc(ident)).unwrap();
-			assert_eq!(atom.data(), data);
-		}
-
 		let mut tag = crate::tag_utils::test_utils::create_tag(TagType::Mp4Ilst);
 
 		tag.insert_text(ItemKey::DiscNumber, String::from("1"));
@@ -552,7 +580,7 @@ mod tests {
 			*b"trkn",
 			&AtomData::Unknown {
 				code: 0,
-				data: vec![0, 0, 0, 1, 0, 0],
+				data: vec![0, 0, 0, 1, 0, 0, 0, 0],
 			},
 		);
 		verify_atom(
@@ -560,8 +588,40 @@ mod tests {
 			*b"disk",
 			&AtomData::Unknown {
 				code: 0,
-				data: vec![0, 0, 0, 1, 0, 2],
+				data: vec![0, 0, 0, 1, 0, 2, 0, 0],
 			},
 		)
 	}
+
+	#[test]
+	fn issue_34() {
+		let ilst = read_ilst("tests/tags/assets/issue_34.ilst");
+
+		verify_atom(
+			&ilst,
+			*b"\xa9ART",
+			&AtomData::UTF8(String::from("Foo artist")),
+		);
+		verify_atom(
+			&ilst,
+			*b"plID",
+			&AtomData::Unknown {
+				code: 21,
+				data: 88888_u64.to_be_bytes().to_vec(),
+			},
+		)
+	}
+
+	#[test]
+	fn advisory_rating() {
+		let ilst = read_ilst("tests/tags/assets/advisory_rating.ilst");
+
+		verify_atom(
+			&ilst,
+			*b"\xa9ART",
+			&AtomData::UTF8(String::from("Foo artist")),
+		);
+
+		assert_eq!(ilst.advisory_rating(), Some(AdvisoryRating::Explicit));
+	}
 }
