@@ -1,6 +1,5 @@
 use super::find_last_page;
-use crate::error::{FileDecodingError, Result};
-use crate::types::file::FileType;
+use crate::error::Result;
 use crate::types::properties::FileProperties;
 
 use std::io::{Read, Seek, SeekFrom};
@@ -10,14 +9,15 @@ use byteorder::{LittleEndian, ReadBytesExt};
 use ogg_pager::Page;
 
 #[derive(Debug, Copy, Clone, PartialEq, Default)]
+#[non_exhaustive]
 /// An Opus file's audio properties
 pub struct OpusProperties {
-	duration: Duration,
-	overall_bitrate: u32,
-	audio_bitrate: u32,
-	channels: u8,
-	version: u8,
-	input_sample_rate: u32,
+	pub(crate) duration: Duration,
+	pub(crate) overall_bitrate: u32,
+	pub(crate) audio_bitrate: u32,
+	pub(crate) channels: u8,
+	pub(crate) version: u8,
+	pub(crate) input_sample_rate: u32,
 }
 
 impl From<OpusProperties> for FileProperties {
@@ -34,25 +34,6 @@ impl From<OpusProperties> for FileProperties {
 }
 
 impl OpusProperties {
-	/// Create a new [`OpusProperties`]
-	pub const fn new(
-		duration: Duration,
-		overall_bitrate: u32,
-		audio_bitrate: u32,
-		channels: u8,
-		version: u8,
-		input_sample_rate: u32,
-	) -> Self {
-		Self {
-			duration,
-			overall_bitrate,
-			audio_bitrate,
-			channels,
-			version,
-			input_sample_rate,
-		}
-	}
-
 	/// Duration
 	pub fn duration(&self) -> Duration {
 		self.duration
@@ -96,15 +77,19 @@ where
 		(end - first_page.start, end)
 	};
 
+	let mut properties = OpusProperties::default();
+
 	let first_page_abgp = first_page.abgp;
 
 	// Skip identification header
 	let first_page_content = &mut &first_page.content()[8..];
 
-	let version = first_page_content.read_u8()?;
-	let channels = first_page_content.read_u8()?;
+	properties.version = first_page_content.read_u8()?;
+	properties.channels = first_page_content.read_u8()?;
+
 	let pre_skip = first_page_content.read_u16::<LittleEndian>()?;
-	let input_sample_rate = first_page_content.read_u32::<LittleEndian>()?;
+
+	properties.input_sample_rate = first_page_content.read_u32::<LittleEndian>()?;
 
 	// Subtract the identification and metadata packet length from the total
 	let audio_size = stream_len - data.seek(SeekFrom::Current(0))?;
@@ -112,30 +97,13 @@ where
 	let last_page = find_last_page(data)?;
 	let last_page_abgp = last_page.abgp;
 
-	last_page_abgp
-		.checked_sub(first_page_abgp + u64::from(pre_skip))
-		.map_or_else(
-			|| {
-				Err(
-					FileDecodingError::new(FileType::Opus, "File contains incorrect PCM values")
-						.into(),
-				)
-			},
-			|frame_count| {
-				let length = frame_count * 1000 / 48000;
-				let duration = Duration::from_millis(length as u64);
+	if let Some(frame_count) = last_page_abgp.checked_sub(first_page_abgp + u64::from(pre_skip)) {
+		let length = frame_count * 1000 / 48000;
+		properties.duration = Duration::from_millis(length as u64);
 
-				let overall_bitrate = ((file_length * 8) / length) as u32;
-				let audio_bitrate = (audio_size * 8 / length) as u32;
+		properties.overall_bitrate = ((file_length * 8) / length) as u32;
+		properties.audio_bitrate = (audio_size * 8 / length) as u32;
+	}
 
-				Ok(OpusProperties {
-					duration,
-					overall_bitrate,
-					audio_bitrate,
-					channels,
-					version,
-					input_sample_rate,
-				})
-			},
-		)
+	Ok(properties)
 }
