@@ -1,10 +1,11 @@
 use crate::error::{ErrorKind, LoftyError, Result};
-use crate::ogg::constants::{OPUSHEAD, VORBIS_IDENT_HEAD};
+use crate::ogg::write::OGGFormat;
 use crate::probe::Probe;
+use crate::tag_traits::{Accessor, TagExt};
 use crate::types::file::FileType;
 use crate::types::item::{ItemKey, ItemValue, TagItem};
 use crate::types::picture::{Picture, PictureInformation, PictureType};
-use crate::types::tag::{Accessor, Tag, TagType};
+use crate::types::tag::{Tag, TagType};
 
 use std::fs::{File, OpenOptions};
 use std::io::{Cursor, Write};
@@ -133,15 +134,21 @@ impl VorbisComments {
 	}
 }
 
-impl VorbisComments {
+impl TagExt for VorbisComments {
+	type Err = LoftyError;
+
+	fn is_empty(&self) -> bool {
+		self.items.is_empty() && self.pictures.is_empty()
+	}
+
 	/// Writes the tag to a path
 	///
 	/// # Errors
 	///
 	/// * `path` does not exist
-	/// * See [`VorbisComments::write_to`]
-	pub fn write_to_path(&self, path: impl AsRef<Path>) -> Result<()> {
-		self.write_to(&mut OpenOptions::new().read(true).write(true).open(path)?)
+	/// * See [`VorbisComments::save_to`]
+	fn save_to_path<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), Self::Err> {
+		self.save_to(&mut OpenOptions::new().read(true).write(true).open(path)?)
 	}
 
 	/// Writes the tag to a file
@@ -150,8 +157,15 @@ impl VorbisComments {
 	///
 	/// * Attempting to write the tag to a format that does not support it
 	/// * The file does not contain valid packets
-	pub fn write_to(&self, file: &mut File) -> Result<()> {
-		Into::<VorbisCommentsRef>::into(self).write_to(file)
+	/// * [`PictureInformation::from_picture`]
+	/// * [`std::io::Error`]
+	fn save_to(&self, file: &mut File) -> std::result::Result<(), Self::Err> {
+		VorbisCommentsRef {
+			vendor: self.vendor.as_str(),
+			items: self.items.iter().map(|(k, v)| (k.as_str(), v.as_str())),
+			pictures: self.pictures.iter().map(|(p, i)| (p, *i)),
+		}
+		.write_to(file)
 	}
 
 	/// Dumps the tag to a writer
@@ -163,8 +177,21 @@ impl VorbisComments {
 	///
 	/// * [`PictureInformation::from_picture`]
 	/// * [`std::io::Error`]
-	pub fn dump_to<W: Write>(&self, writer: &mut W) -> Result<()> {
-		Into::<VorbisCommentsRef>::into(self).dump_to(writer)
+	fn dump_to<W: Write>(&self, writer: &mut W) -> std::result::Result<(), Self::Err> {
+		VorbisCommentsRef {
+			vendor: self.vendor.as_str(),
+			items: self.items.iter().map(|(k, v)| (k.as_str(), v.as_str())),
+			pictures: self.pictures.iter().map(|(p, i)| (p, *i)),
+		}
+		.dump_to(writer)
+	}
+
+	fn remove_from_path<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), Self::Err> {
+		TagType::VorbisComments.remove_from_path(path)
+	}
+
+	fn remove_from(&self, file: &mut File) -> std::result::Result<(), Self::Err> {
+		TagType::VorbisComments.remove_from(file)
 	}
 }
 
@@ -172,11 +199,6 @@ impl From<VorbisComments> for Tag {
 	fn from(input: VorbisComments) -> Self {
 		let mut tag = Tag::new(TagType::VorbisComments);
 
-		tag.items.push(TagItem::new(
-			ItemKey::EncoderSoftware,
-			ItemValue::Text(input.vendor),
-		));
-
 		for (k, v) in input.items {
 			tag.items.push(TagItem::new(
 				ItemKey::from_key(TagType::VorbisComments, &k),
@@ -184,6 +206,18 @@ impl From<VorbisComments> for Tag {
 			));
 		}
 
+		// We need to preserve the vendor string
+		if !tag
+			.items
+			.iter()
+			.any(|i| i.key() == &ItemKey::EncoderSoftware)
+		{
+			tag.items.push(TagItem::new(
+				ItemKey::EncoderSoftware,
+				ItemValue::Text(input.vendor),
+			));
+		}
+
 		for (pic, _info) in input.pictures {
 			tag.push_picture(pic)
 		}
@@ -231,13 +265,21 @@ impl From<Tag> for VorbisComments {
 	}
 }
 
-pub(crate) struct VorbisCommentsRef<'a> {
+pub(crate) struct VorbisCommentsRef<'a, II, IP>
+where
+	II: Iterator<Item = (&'a str, &'a str)>,
+	IP: Iterator<Item = (&'a Picture, PictureInformation)>,
+{
 	pub vendor: &'a str,
-	pub items: Box<dyn Iterator<Item = (&'a str, &'a str)> + 'a>,
-	pub pictures: Box<dyn Iterator<Item = (&'a Picture, PictureInformation)> + 'a>,
+	pub items: II,
+	pub pictures: IP,
 }
 
-impl<'a> VorbisCommentsRef<'a> {
+impl<'a, II, IP> VorbisCommentsRef<'a, II, IP>
+where
+	II: Iterator<Item = (&'a str, &'a str)>,
+	IP: Iterator<Item = (&'a Picture, PictureInformation)>,
+{
 	#[allow(clippy::shadow_unrelated)]
 	fn write_to(&mut self, file: &mut File) -> Result<()> {
 		let probe = Probe::new(file).guess_file_type()?;
@@ -247,68 +289,50 @@ impl<'a> VorbisCommentsRef<'a> {
 
 		match f_ty {
 			Some(FileType::FLAC) => super::flac::write::write_to(file, self),
-			Some(FileType::Opus) => super::write::write(file, self, OPUSHEAD),
-			Some(FileType::Vorbis) => super::write::write(file, self, VORBIS_IDENT_HEAD),
+			Some(FileType::Opus) => super::write::write(file, self, OGGFormat::Opus),
+			Some(FileType::Vorbis) => super::write::write(file, self, OGGFormat::Vorbis),
+			Some(FileType::Speex) => super::write::write(file, self, OGGFormat::Speex),
 			_ => Err(LoftyError::new(ErrorKind::UnsupportedTag)),
 		}
 	}
 
 	pub(crate) fn dump_to<W: Write>(&mut self, writer: &mut W) -> Result<()> {
 		let mut temp = Cursor::new(Vec::new());
-		super::write::create_pages(self, &mut temp)?;
+		super::write::create_pages(self, &mut temp, 0, false)?;
 
 		writer.write_all(temp.get_ref())?;
 		Ok(())
 	}
 }
 
-impl<'a> Into<VorbisCommentsRef<'a>> for &'a VorbisComments {
-	fn into(self) -> VorbisCommentsRef<'a> {
-		VorbisCommentsRef {
-			vendor: self.vendor.as_str(),
-			items: Box::new(
-				self.items
-					.as_slice()
-					.iter()
-					.map(|(k, v)| (k.as_str(), v.as_str())),
-			),
-			pictures: Box::new(self.pictures.as_slice().iter().map(|(p, i)| (p, *i))),
-		}
-	}
-}
+pub(crate) fn create_vorbis_comments_ref(
+	tag: &Tag,
+) -> (
+	&str,
+	impl Iterator<Item = (&str, &str)>,
+	impl Iterator<Item = (&Picture, PictureInformation)>,
+) {
+	let vendor = tag.get_string(&ItemKey::EncoderSoftware).unwrap_or("");
 
-impl<'a> Into<VorbisCommentsRef<'a>> for &'a Tag {
-	fn into(self) -> VorbisCommentsRef<'a> {
-		let vendor = self.get_string(&ItemKey::EncoderSoftware).unwrap_or("");
+	let items = tag.items.iter().filter_map(|i| match i.value() {
+		ItemValue::Text(val) | ItemValue::Locator(val) => i
+			.key()
+			.map_key(TagType::VorbisComments, true)
+			.map(|key| (key, val.as_str())),
+		_ => None,
+	});
 
-		let items = self.items.iter().filter_map(|i| match i.value() {
-			ItemValue::Text(val) | ItemValue::Locator(val)
-				// Already got the vendor above
-				if i.key() != &ItemKey::EncoderSoftware =>
-			{
-				i.key()
-					.map_key(TagType::VorbisComments, true)
-					.map(|key| (key, val.as_str()))
-			},
-			_ => None,
-		});
-
-		VorbisCommentsRef {
-			vendor,
-			items: Box::new(items),
-			pictures: Box::new(
-				self.pictures
-					.iter()
-					.map(|p| (p, PictureInformation::default())),
-			),
-		}
-	}
+	let pictures = tag
+		.pictures
+		.iter()
+		.map(|p| (p, PictureInformation::default()));
+	(vendor, items, pictures)
 }
 
 #[cfg(test)]
 mod tests {
 	use crate::ogg::VorbisComments;
-	use crate::{Tag, TagType};
+	use crate::{Tag, TagExt, TagType};
 
 	use std::io::Read;
 
