@@ -1,27 +1,23 @@
-#[cfg(feature = "ape")]
-use crate::ape::tag::ape_tag::ApeTagRef;
 use crate::error::{ErrorKind, LoftyError, Result};
-#[cfg(feature = "id3v1")]
-use crate::id3::v1::tag::Id3v1TagRef;
-#[cfg(feature = "id3v2")]
-use crate::id3::v2::{
-	tag::{tag_frames, Id3v2TagRef},
-	Id3v2TagFlags,
-};
-#[cfg(feature = "aiff_text_chunks")]
-use crate::iff::aiff::tag::AiffTextChunksRef;
-#[cfg(feature = "riff_info_list")]
-use crate::iff::wav::tag::RiffInfoListRef;
-#[cfg(feature = "mp4_ilst")]
-use crate::mp4::ilst::IlstRef;
-#[cfg(feature = "vorbis_comments")]
-use crate::ogg::tag::VorbisCommentsRef;
-
-use crate::{ape, iff, mp3, mp4, ogg};
-
 use crate::types::file::FileType;
 use crate::types::item::ItemKey;
 use crate::types::tag::{Tag, TagType};
+use crate::{ape, iff, mp3, mp4, ogg};
+
+#[cfg(feature = "id3v1")]
+use crate::id3::v1::tag::Id3v1TagRef;
+#[cfg(feature = "id3v2")]
+use crate::id3::v2::{self, tag::Id3v2TagRef, Id3v2TagFlags};
+#[cfg(feature = "ape")]
+use ape::tag::ape_tag::ApeTagRef;
+#[cfg(feature = "aiff_text_chunks")]
+use iff::aiff::tag::AiffTextChunksRef;
+#[cfg(feature = "riff_info_list")]
+use iff::wav::tag::RiffInfoListRef;
+#[cfg(feature = "mp4_ilst")]
+use mp4::ilst::IlstRef;
+#[cfg(feature = "vorbis_comments")]
+use ogg::tag::{create_vorbis_comments_ref, VorbisCommentsRef};
 
 use std::fs::File;
 use std::io::Write;
@@ -32,14 +28,12 @@ pub(crate) fn write_tag(tag: &Tag, file: &mut File, file_type: FileType) -> Resu
 		FileType::AIFF => iff::aiff::write::write_to(file, tag),
 		FileType::APE => ape::write::write_to(file, tag),
 		#[cfg(feature = "vorbis_comments")]
-		FileType::FLAC => ogg::flac::write::write_to(file, &mut Into::<VorbisCommentsRef>::into(tag)),
+		FileType::FLAC | FileType::Opus | FileType::Speex | FileType::Vorbis => {
+			ogg::write::write_to(file, tag, file_type)
+		},
 		FileType::MP3 => mp3::write::write_to(file, tag),
 		#[cfg(feature = "mp4_ilst")]
-		FileType::MP4 => mp4::ilst::write::write_to(file, &mut Into::<IlstRef>::into(tag)),
-		#[cfg(feature = "vorbis_comments")]
-		FileType::Opus => ogg::write::write_to(file, tag, ogg::constants::OPUSTAGS),
-		#[cfg(feature = "vorbis_comments")]
-		FileType::Vorbis => ogg::write::write_to(file, tag, ogg::constants::VORBIS_COMMENT_HEAD),
+		FileType::MP4 => mp4::ilst::write::write_to(file, &mut Into::<IlstRef<'_>>::into(tag)),
 		FileType::WAV => iff::wav::write::write_to(file, tag),
 		_ => Err(LoftyError::new(ErrorKind::UnsupportedTag)),
 	}
@@ -49,64 +43,50 @@ pub(crate) fn write_tag(tag: &Tag, file: &mut File, file_type: FileType) -> Resu
 pub(crate) fn dump_tag<W: Write>(tag: &Tag, writer: &mut W) -> Result<()> {
 	match tag.tag_type() {
 		#[cfg(feature = "ape")]
-		TagType::Ape => Into::<ApeTagRef>::into(tag).dump_to(writer),
+		TagType::Ape => ApeTagRef {
+			read_only: false,
+			items: ape::tag::ape_tag::tagitems_into_ape(tag.items()),
+		}
+		.dump_to(writer),
 		#[cfg(feature = "id3v1")]
-		TagType::Id3v1 => Into::<Id3v1TagRef>::into(tag).dump_to(writer),
+		TagType::Id3v1 => Into::<Id3v1TagRef<'_>>::into(tag).dump_to(writer),
 		#[cfg(feature = "id3v2")]
-		TagType::Id3v2 => Id3v2TagRef::new(Id3v2TagFlags::default(), tag_frames(tag)).dump_to(writer),
+		TagType::Id3v2 => Id3v2TagRef {
+			flags: Id3v2TagFlags::default(),
+			frames: v2::tag::tag_frames(tag),
+		}
+		.dump_to(writer),
 		#[cfg(feature = "mp4_ilst")]
-		TagType::Mp4Ilst => Into::<IlstRef>::into(tag).dump_to(writer),
+		TagType::Mp4Ilst => Into::<IlstRef<'_>>::into(tag).dump_to(writer),
 		#[cfg(feature = "vorbis_comments")]
-		TagType::VorbisComments => Into::<VorbisCommentsRef>::into(tag).dump_to(writer),
+		TagType::VorbisComments => {
+			let (vendor, items, pictures) = create_vorbis_comments_ref(tag);
+
+			VorbisCommentsRef {
+				vendor,
+				items,
+				pictures,
+			}
+			.dump_to(writer)
+		},
 		#[cfg(feature = "riff_info_list")]
-		TagType::RiffInfo => Into::<RiffInfoListRef>::into(tag).dump_to(writer),
+		TagType::RiffInfo => RiffInfoListRef {
+			items: iff::wav::tag::tagitems_into_riff(tag.items()),
+		}
+		.dump_to(writer),
 		#[cfg(feature = "aiff_text_chunks")]
-		TagType::AiffText => AiffTextChunksRef::new(
-			tag.get_string(&ItemKey::TrackTitle),
-			tag.get_string(&ItemKey::TrackArtist),
-			tag.get_string(&ItemKey::CopyrightMessage),
-			Some(tag.get_texts(&ItemKey::Comment)),
-			None,
-		)
+		TagType::AiffText => AiffTextChunksRef {
+			name: tag.get_string(&ItemKey::TrackTitle),
+			author: tag.get_string(&ItemKey::TrackArtist),
+			copyright: tag.get_string(&ItemKey::CopyrightMessage),
+			annotations: Some(tag.get_texts(&ItemKey::Comment)),
+			comments: None,
+		}
 		.dump_to(writer),
 		_ => Ok(()),
 	}
 }
 
-macro_rules! tag_methods {
-	(
-		$(
-			$(#[$attr:meta])?;
-			$name:ident,
-			$ty:ty
-		);*
-	) => {
-		paste::paste! {
-			$(
-				$(#[$attr])?
-				#[doc = "Gets the [`" $ty "`] if it exists"]
-				pub fn $name(&self) -> Option<&$ty> {
-					self.$name.as_ref()
-				}
-
-				$(#[$attr])?
-				#[doc = "Gets a mutable reference to the [`" $ty "`] if it exists"]
-				pub fn [<$name _mut>](&mut self) -> Option<&mut $ty> {
-					self.$name.as_mut()
-				}
-
-				$(#[$attr])?
-				#[doc = "Removes the [`" $ty "`]"]
-				pub fn [<remove_ $name>](&mut self) {
-					self.$name = None
-				}
-			)*
-		}
-	}
-}
-
-pub(crate) use tag_methods;
-
 #[cfg(test)]
 // Used for tag conversion tests
 pub(crate) mod test_utils {
