@@ -1,16 +1,13 @@
 use crate::types::tag::TagType;
 
-#[allow(unused_imports)]
 use std::collections::HashMap;
 
-#[allow(unused)]
 macro_rules! first_key {
 	($key:tt $(| $remaining:expr)*) => {
 		$key
 	};
 }
 
-#[allow(unused)]
 pub(crate) use first_key;
 
 // This is used to create the key/ItemKey maps
@@ -69,9 +66,9 @@ gen_map!(
 	#[cfg(feature = "aiff_text_chunks")]
 	AIFF_TEXT_MAP;
 
-	"NAME" 			=> TrackTitle,
-	"AUTH" 			=> TrackArtist,
-	"(c) " 			=> CopyrightMessage,
+	"NAME"			=> TrackTitle,
+	"AUTH"			=> TrackArtist,
+	"(c) "			=> CopyrightMessage,
 	"COMM" | "AUTH" => Comment
 );
 
@@ -79,15 +76,15 @@ gen_map!(
 	#[cfg(feature = "ape")]
 	APE_MAP;
 
-	"Album" 	   	 			   => AlbumTitle,
-	"DiscSubtitle" 	   			   => SetSubtitle,
+	"Album" 	   	  			   => AlbumTitle,
+	"DiscSubtitle" 	  			   => SetSubtitle,
 	"Grouping"	  	  			   => ContentGroup,
 	"Title"		   	  			   => TrackTitle,
 	"Subtitle"	   	  			   => TrackSubtitle,
 	"ALBUMSORT"	   	  			   => AlbumTitleSortOrder,
 	"ALBUMARTISTSORT" 			   => AlbumArtistSortOrder,
-	"TITLESORT"					   => TrackTitleSortOrder,
-	"ARTISTSORT"	 			   => TrackArtistSortOrder,
+	"TITLESORT"		  			   => TrackTitleSortOrder,
+	"ARTISTSORT"	  			   => TrackArtistSortOrder,
 	"Album Artist" | "ALBUMARTIST" => AlbumArtist,
 	"Artist"					   => TrackArtist,
 	"Arranger"					   => Arranger,
@@ -126,10 +123,10 @@ gen_map! (
 	#[cfg(feature = "id3v2")]
 	ID3V2_MAP;
 
-	"TALB" 			=> AlbumTitle,
-	"TSST" 			=> SetSubtitle,
+	"TALB"			=> AlbumTitle,
+	"TSST"			=> SetSubtitle,
 	"TIT1" | "GRP1" => ContentGroup,
-	"TIT2"			=> TrackTitle,
+	"TIT2" 			=> TrackTitle,
 	"TIT3" 			=> TrackSubtitle,
 	"TOAL" 			=> OriginalAlbumTitle,
 	"TOPE" 			=> OriginalArtist,
@@ -203,10 +200,10 @@ gen_map! (
 	#[cfg(feature = "mp4_ilst")]
 	ILST_MAP;
 
-	"\u{a9}alb" 						  => AlbumTitle,
+	"\u{a9}alb"							  => AlbumTitle,
 	"----:com.apple.iTunes:DISCSUBTITLE"  => SetSubtitle,
-	"tvsh" 								  => ShowName,
-	"\u{a9}grp"						      => ContentGroup,
+	"tvsh"								  => ShowName,
+	"\u{a9}grp"							  => ContentGroup,
 	"\u{a9}nam"							  => TrackTitle,
 	"----:com.apple.iTunes:SUBTITLE"	  => TrackSubtitle,
 	"soal"								  => AlbumTitleSortOrder,
@@ -230,7 +227,8 @@ gen_map! (
 	"disk"								  => DiscTotal,
 	"trkn"								  => TrackNumber,
 	"trkn"								  => TrackTotal,
-	"rate"								  => LawRating,
+	"rate"								  => Popularimeter,
+	"rtng"								  => ParentalAdvisory,
 	"\u{a9}day"							  => RecordingDate,
 	"----:com.apple.iTunes:ISRC"		  => ISRC,
 	"----:com.apple.iTunes:BARCODE"		  => Barcode,
@@ -269,7 +267,7 @@ gen_map! (
 	"IPRO" 			=> Producer,
 	"IPRT" | "ITRK" => TrackNumber,
 	"IFRM" 			=> TrackTotal,
-	"IRTD" 			=> LawRating,
+	"IRTD" 			=> Popularimeter,
 	"ICRD" 			=> RecordingDate,
 	"TLEN"			=> Length,
 	"ISRF" 			=> OriginalMediaType,
@@ -285,36 +283,37 @@ gen_map!(
 	#[cfg(feature = "vorbis_comments")]
 	VORBIS_MAP;
 
-	"ALBUM" 	      		   	   => AlbumTitle,
-	"DISCSUBTITLE"    		   	   => SetSubtitle,
-	"GROUPING"	   	  		   	   => ContentGroup,
-	"TITLE"		   	  		   	   => TrackTitle,
-	"SUBTITLE"	   	  		   	   => TrackSubtitle,
-	"ALBUMSORT"	   	  		   	   => AlbumTitleSortOrder,
-	"ALBUMARTISTSORT" 		   	   => AlbumArtistSortOrder,
-	"TITLESORT" 	  		   	   => TrackTitleSortOrder,
-	"ARTISTSORT"	  		   	   => TrackArtistSortOrder,
-	"ALBUMARTIST"	  		   	   => AlbumArtist,
-	"ARTIST"		  		   	   => TrackArtist,
-	"ARRANGER"		  		   	   => Arranger,
-	"AUTHOR" | "WRITER" 	   	   => Writer,
-	"COMPOSER"				   	   => Composer,
-	"CONDUCTOR"				   	   => Conductor,
-	"ENGINEER"				   	   => Engineer,
-	"LYRICIST"				   	   => Lyricist,
-	"DJMIXER"				   	   => MixDj,
-	"MIXER"					   	   => MixEngineer,
-	"PERFORMER"				   	   => Performer,
-	"PRODUCER"				   	   => Producer,
-	"PUBLISHER"				   	   => Publisher,
-	"LABEL"					   	   => Label,
-	"REMIXER"				   	   => Remixer,
-	"DISCNUMBER"			   	   => DiscNumber,
-	"DISCTOTAL" | "TOTALDISCS" 	   => DiscTotal,
-	"TRACKNUMBER"			   	   => TrackNumber,
+	"ALBUM" 	      			   => AlbumTitle,
+	"DISCSUBTITLE"    			   => SetSubtitle,
+	"GROUPING"	   	  			   => ContentGroup,
+	"TITLE"		   	  			   => TrackTitle,
+	"SUBTITLE"	   	  			   => TrackSubtitle,
+	"ALBUMSORT"	   	  			   => AlbumTitleSortOrder,
+	"ALBUMARTISTSORT" 			   => AlbumArtistSortOrder,
+	"TITLESORT" 	  			   => TrackTitleSortOrder,
+	"ARTISTSORT"	  			   => TrackArtistSortOrder,
+	"ALBUMARTIST"	  			   => AlbumArtist,
+	"ARTIST"		  			   => TrackArtist,
+	"ARRANGER"		  			   => Arranger,
+	"AUTHOR" | "WRITER" 		   => Writer,
+	"COMPOSER"					   => Composer,
+	"CONDUCTOR"					   => Conductor,
+	"ENGINEER"					   => Engineer,
+	"LYRICIST"					   => Lyricist,
+	"DJMIXER"					   => MixDj,
+	"MIXER"						   => MixEngineer,
+	"PERFORMER"					   => Performer,
+	"PRODUCER"					   => Producer,
+	"PUBLISHER"					   => Publisher,
+	"LABEL"						   => Label,
+	"REMIXER"					   => Remixer,
+	"DISCNUMBER"				   => DiscNumber,
+	"DISCTOTAL" | "TOTALDISCS"	   => DiscTotal,
+	"TRACKNUMBER"				   => TrackNumber,
 	"TRACKTOTAL" | "TOTALTRACKS"   => TrackTotal,
+	"RATING"					   => Popularimeter,
 	"DATE"						   => RecordingDate,
-	"YEAR" 			   			   => Year,
+	"YEAR" 						   => Year,
 	"ORIGINALDATE" 				   => OriginalReleaseDate,
 	"ISRC" 						   => ISRC,
 	"CATALOGNUMBER" 			   => CatalogNumber,
@@ -324,8 +323,8 @@ gen_map!(
 	"ENCODER" 					   => EncoderSoftware,
 	"ENCODING" | "ENCODERSETTINGS" => EncoderSettings,
 	"GENRE" 					   => Genre,
-	"MOOD" 					 	   => Mood,
-	"BPM" 					 	   => BPM,
+	"MOOD" 						   => Mood,
+	"BPM" 						   => BPM,
 	"COPYRIGHT" 				   => CopyrightMessage,
 	"LICENSE" 					   => License,
 	"COMMENT" 					   => Comment,
@@ -471,7 +470,7 @@ gen_item_keys!(
 		TrackNumber,
 		TrackTotal,
 		Popularimeter,
-		LawRating,
+		ParentalAdvisory,
 
 		// Dates
 		RecordingDate,
